<?php
/**
 * @file
 * iToggle ajax functions.
 */

/**
 * Page callback.
 * iToggle widget AJAX callback.
 *
 * @see itoggle_menu()
 */
function itoggle_ajax_callback() {
  // Extract values from POST.
  $token = filter_input(INPUT_POST, 'token');
  $type = filter_input(INPUT_POST, 'type');
  $property = filter_input(INPUT_POST, 'property');
  $id = filter_input(INPUT_POST, 'id');
  $value = filter_input(INPUT_POST, 'value');
  $scope = filter_input(INPUT_POST, 'scope');
  // Generate token key.
  $token_key = "itoggle_{$type}_{$property}_{$id}";

  // Route request.
  if (!empty($token) && drupal_valid_token($token, $token_key)) {
    switch ($scope) {
      case 'entity':
        $ok = _itoggle_ajax_toggle_entity($type, $property, $id, $value);
        break;
      case 'field':
        $ok = _itoggle_ajax_toggle_field($type, $property, $id, $value);
        break;
      // Should never really get here.
      case 'field-edit':
        $ok = TRUE;
        break;
    }
  }
  else {
    // Let $ok be NULL so we know the difference between FALSE (error) and
    // UNDEFINDED (invalid token).
    $ok = NULL;
  }

  drupal_json_output(array('ok' => $ok));
  exit;
}

/**
 * Helper function.
 * Change field values.
 *
 * @param string $type
 * @param string $property
 * @param int $id
 * @param int $value
 * @param int $delta
 */
function _itoggle_ajax_toggle_field($type, $property, $id, $value, $delta = 0) {
  $access = FALSE;

  // Check permissions
  // @TODO some other way of controlling permissions for other bundles
  if ($type === 'node') {
    global $user;
    $node = node_load($id);

    if ($node->uid == $user->uid) {
      $access = user_access("edit own {$type} content", $user);
    }
    else {
      $access = user_access("edit any {$type} content", $user);
    }
  }

  if ($access) {
    $entity = current(entity_load($type, array($id)));

    if ($entity) {
//      $lang = $entity->language;
      $lang = LANGUAGE_NONE;
      $entity->{$property}[$lang][$delta] = $value;
      $return = entity_save($type, $entity);
      return $return !== FALSE;
    }
  }

  return FALSE;
}

/**
 * Helper function.
 * Changes entity properties.
 *
 * @param string $type
 * @param string $property
 * @param int $id
 * @param int $value
 */
function _itoggle_ajax_toggle_entity($type, $property, $id, $value) {
  $access = FALSE;

  // Check permissions.
  if ($type === 'node') {
    $access = user_access('bypass node access') || user_access('administer nodes');
  }

  if (!$access && $type === 'node' && module_exists('override_node_options')) {
    $node = node_load($id);
    $type = $node->type;
    $prop = ($property === 'status') ? 'published' : $property;
    $access = user_access("override {$type} {$prop} option");
  }

  if ($access === TRUE) {
    if ($entity = entity_load_single($type, $id)) {
      $entity->{$property} = $value;
      return entity_save($type, $entity) !== FALSE;
    }
  }

  return FALSE;
}